#!/usr/bin/env Rscript

################################################
#### Load required library
################################################

library('sleuth')
library("jsonlite")

################################################
#### Read arguments from CLI
################################################

args <- commandArgs(trailingOnly = T)

inKallistoResults <- args[1]
inMetadataFile <- args[2]
inMetadataSample <- args[3]
inMetadataFactorList <- args[4]
inMetadataDataCol <- args[5]
outFolder <- args[6]

# inKallistoResults <- "../../runs/kallisto/kallisto_results"
# inMetadataFile <- "../../runs/isaSampleToRawDataSeq/rnaseq-samples.csv"
# inMetadataSample <- "Input [Source Name]"
# inMetadataFactorList <- list("Factor [Photosynthesis mode]")
# inMetadataDataCol <- "Output [Data]"
# outFolder <- "."

################################################
#### If it does not exist, create out dir
################################################

dir.create(outFolder, recursive = T, showWarnings = F)

################################################
#### Read ISA sample metadata
################################################

samples <- read.csv(file = inMetadataFile, check.names = FALSE)

################################################
#### Read Kallisto results
################################################

base_dir <- inKallistoResults

# A list of paths to the kallisto results indexed by the sample IDs is collated with
kal_dirs <- dir(base_dir, full.names = T) ## Sleuth requires full paths

s2c <- samples[order(samples[[inMetadataSample]]), c(inMetadataSample, unlist(inMetadataFactorList), inMetadataDataCol)]

# For kallisto / sleuth: 's2c' (sample_to_covariates) must contain a column named 'sample'
colnames(s2c)[1] <- c("sample")
path_df <- data.frame(path = kal_dirs, sample = basename(kal_dirs))
s2c <- merge(s2c, path_df, by = "sample")

# Build a sleuth object

factors <- sapply(inMetadataFactorList, function(x) x[[1]])
## Annoying workaround to prevent formula error with special chars in column headers
colnames(s2c) <- make.names(colnames(s2c))
factors <- make.names(factors)

design_formula <- as.formula(paste("~", paste(rev(factors), collapse = " + ")))

so <- sleuth_prep(s2c, full_model = design_formula)

save(so, file = file.path(outFolder, "kallisto_sleuthObject.RData"))

################################################
#### Extract expression tables
################################################

## as data.frame
expression_data <- kallisto_table(so)

## write to file
write.csv(expression_data, file.path(outFolder, "kallisto_df.csv"), row.names = F)

## as tpm matrix (gene x sample)
tpm_table <- reshape(expression_data, idvar = "target_id", timevar = "sample", direction = "wide", v.names = "tpm")

# Write to file
write.csv(tpm_table, file.path(outFolder, "kallisto_tpmMatrix.csv"), row.names = F)

################################################
#### Summarize mapping stats
################################################

mapping_stats <- list()
for (i in dir(kal_dirs, pattern = ".json", full.names = T)) {
  id <- unlist(strsplit(i, split = "/"))
  z <- data.frame(ID = id[length(id) - 1], fromJSON(i, simplifyVector = T))
  mapping_stats <- rbind(mapping_stats, z)
}

write.csv(mapping_stats, file.path(outFolder, "kallisto_mappingStats.csv"), row.names = F)


################################################
#### Run sleuth to identify DEGs
################################################

so <- sleuth_fit(so)
so <- sleuth_fit(so, design_formula, "full")
so <- sleuth_fit(so, ~1, "reduced")
so <- sleuth_lrt(so, "reduced", "full")

sleuth_table <- sleuth_results(so, "reduced:full", "lrt", show_all = FALSE)

### write to file

write.csv(sleuth_table, file.path(outFolder, "sleuth_dge.csv"), row.names = F)
